library(dplyr)
library(devtools)
library(pca3d)
library(rgl)
library(scatterplot3d)
library(mvnormtest)

# Data Analysis R Script: 
# Acoustic Niches in the Brazilian Atlantic Forest

# # In this script, the two study sites (Aleixo & Santa Marta) are
# # commonly denoted with abbreviations of A and SM 

#####################################

# Step 1: Reading in the data of species means from each study site
soundA <- read.csv("/Volumes/TRISTAN/Dissertation/dataALEIXO.csv")
soundSM <- read.csv("/Volumes/TRISTAN/Dissertation/dataSANTAMARTA.csv")

# Step 2: Removing species with only one observed song and removing secondary songs of species
soundA <- filter(soundA, !soundA$Songs<2)
soundSM <- filter(soundSM, !soundSM$Songs<2)

# Step 3: Producing the principal component analysis
soundA.pca <- prcomp(soundA[,c(2:8)], center = TRUE, scale. = TRUE)
soundSM.pca <- prcomp(soundSM[,c(2:8)], center = TRUE, scale. = TRUE)
summary(soundA.pca)
summary(soundSM.pca)

# Step 4: Producing 2D and 3D PCA plots

# # Using the pca3d package for interactive visualization 
# # of the acoustic space of each study site

pca3d(soundA.pca, col = "green", show.plane = TRUE)
open3d()
pca3d(soundSM.pca, col = "blue", show.plane = TRUE)

par(mfrow=c(1,1)) 
pca2d(soundA.pca, col = "black")
pca2d(soundSM.pca, col = "black")

# Step 5: Extracting the coordinates of each birdsong in PCA space

# # This step is necessary for later calculations of the acoustic distances
# # between species pairs and also for an alternative method of producing
# # 3D and 2D plots of the acoustic space.

pcaAresults <- data.frame(soundA.pca$x)
pcaSMresults <- data.frame(soundSM.pca$x)
pcaAresults <- mutate(pcaAresults, species = soundA$Species) %>% select(-c(PC5, PC6, PC7))
pcaSMresults <- mutate(pcaSMresults, species = soundSM$Species) %>% select(-c(PC5, PC6, PC7))

pcaA_x <- unlist(pcaAresults[1]) 
pcaA_x <- as.double(pcaA_x)
pcaA_y <- unlist(pcaAresults[2])
pcaA_y <- as.double(pcaA_y)
pcaA_z <- unlist(pcaAresults[3]) 
pcaA_z <- as.double(pcaA_z)

meansongA <- c(mean(pcaA_x), mean(pcaA_y), mean(pcaA_z))

# # Below, I also extract the fourth PC axis (pca_d). This fourth axis of 
# # variation is to be used for acoustic distance calculations but not 
# # for three-dimensional plots of the acoustic space.

pcaA_d <- unlist(pcaAresults[4]) 
pcaA_d <- as.double(pcaA_d)

pcaSM_x <- unlist(pcaSMresults[1]) 
pcaSM_x <- as.double(pcaSM_x)
pcaSM_y <- unlist(pcaSMresults[2])
pcaSM_y <- as.double(pcaSM_y)
pcaSM_z <- unlist(pcaSMresults[3])
pcaSM_z <- as.double(pcaSM_z)
pcaSM_d <- unlist(pcaSMresults[4])
pcaSM_d <- as.double(pcaSM_d)

# Step 6: Producing acoustic distance (AD) matrices for each study site

AD_A <- numeric()
AD_SM <- numeric()

# # The below loops create vectors of the mean AD (or nearest
# # neighbour distance) for each species in each study site
for (i in 1:57) {
  x <- mean(sqrt((pcaAresults[1] - pcaAresults$PC1[[i]])^2 + (pcaAresults[2] - pcaAresults$PC2[[i]])^2 + (pcaAresults[3] - pcaAresults$PC3[[i]])^2 + (pcaAresults[4] - pcaAresults$PC4[[i]])^2))
  AD_A <- append(AD_A, x)
} # Aleixo
for (i in 1:29) {
  x <- mean(sqrt((pcaSMresults[1] - pcaSMresults$PC1[[i]])^2 + (pcaSMresults[2] - pcaSMresults$PC2[[i]])^2 + (pcaSMresults[3] - pcaSMresults$PC3[[i]])^2 + (pcaSMresults[4] - pcaSMresults$PC4[[i]])^2))
  AD_SM <- append(AD_SM, x)
} # Santa Marta

acoustic_distances_A <- data.frame(AD_A, soundA$Species)
acoustic_distances_SM <- data.frame(AD_SM, soundSM$Species)
names(acoustic_distances_A) <- c("Mean AD", "Species")
names(acoustic_distances_SM) <- c("Mean_AD", "Species")

# # Finding distance from each birdsong to the centre (mean bird song) of the PCA and putting it in a vector
newpcaA_x <- pcaA_x[-(which(occsumsA<5))]
newpcaA_y <- pcaA_y[-(which(occsumsA<5))]
newpcaA_z <- pcaA_z[-(which(occsumsA<5))]
meandistanceA <- numeric()

for (i in 1:57) {
  x <- sqrt(pcaA_x[i]^2+pcaA_y[i]^2+pcaA_z[i]^2)
  meandistanceA <- append(meandistanceA, x)
}

newpcaSM_x <- pcaSM_x[-(which(occsumsSM<3))]
newpcaSM_y <- pcaSM_y[-(which(occsumsSM<3))]
newpcaSM_z <- pcaSM_z[-(which(occsumsSM<3))]
meandistanceSM <- numeric()

for (i in 1:29) {
  x <- sqrt(pcaSM_x[i]^2+pcaSM_y[i]^2+pcaSM_z[i]^2)
  meandistanceSM <- append(meandistanceSM, x)
}

# # The AD_A_maker and AD_SM_maker loops below calculate all the
# # distances between each species pairing in acoustic space.

# # The two loops afterward simply place these values into matrices.

AD_A_maker <- function(x) {
  vector <- numeric()
  for (i in 1:57) {
    D <- sqrt((pcaAresults$PC1[[x]] - pcaAresults$PC1[[i]])^2 + (pcaAresults$PC2[[x]] - pcaAresults$PC2[[i]])^2 + (pcaAresults$PC3[[x]] - pcaAresults$PC3[[i]])^2 + (pcaAresults$PC4[[x]] - pcaAresults$PC4[[i]])^2)
    vector <- append(vector, D)
  }
  return(vector)
} # Aleixo

AD_SM_maker <- function(x) {
  vector <- numeric()
  for (i in 1:29) {
    D <- sqrt((pcaSMresults$PC1[[x]] - pcaSMresults$PC1[[i]])^2 + (pcaSMresults$PC2[[x]] - pcaSMresults$PC2[[i]])^2 + (pcaSMresults$PC3[[x]] - pcaSMresults$PC3[[i]])^2 + (pcaSMresults$PC4[[x]] - pcaSMresults$PC4[[i]])^2)
    vector <- append(vector, D)
  }
  return(vector)
} # Santa Marta

ADmatrix_A <- data.frame(x = rep(0,57))
ADmatrix_SM <- data.frame(x = rep(0,29))

for (y in 1:57) {
  ADmatrix_A[,y] <- AD_A_maker(y)
}
for (y in 1:29) {
  ADmatrix_SM[,y] <- AD_SM_maker(y)
}

names(ADmatrix_A) <- soundA$Species
names(ADmatrix_SM) <- soundSM$Species

# Step 7: Calculating mean acoustic distance of the entire acoustic community

# # In this short step, the mean and st. deviation are calculated from 
# # the acoustic distance matrices of each site.

ADmatrix_A_vec <- unlist(ADmatrix_A) 
ADmatrix_A_vec <- unique(ADmatrix_A_vec[ADmatrix_A_vec!=0])
ADmatrix_SM_vec <- unlist(ADmatrix_SM) 
ADmatrix_SM_vec <- unique(ADmatrix_SM_vec[ADmatrix_SM_vec!=0])

meanAD_A <- mean(ADmatrix_A_vec)
sdAD_A <- sd(ADmatrix_A_vec)
meanAD_SM <- mean(ADmatrix_SM_vec)
sdAD_SM <- sd(ADmatrix_SM_vec)

# # Below is the same as the first part of Step 5
avgAD_A <- data.frame(soundA$Species, AD_A)
avgAD_SM <- data.frame(soundSM$Species, AD_SM)

# Step 8: Calculating proximity to the centre of acoustic space
fromcentre_A <- sqrt(pcaA_x^2 + pcaA_y^2 + pcaA_z^2)
fromcentre_SM <- sqrt(pcaSM_x^2 + pcaSM_y^2 + pcaSM_z^2)

# Step 9: Finding furthest outliers
soundA$Species[which.max(fromcentre_A)]
soundSM$Species[which.max(fromcentre_SM)]

# Step 10: Calculating density of each site's song distribution
densityA <- length(soundA$Species)/(4*pi*(max(fromcentre_A))^3)/3
densitySM <- length(soundSM$Species)/(4*pi*(max(fromcentre_SM))^3)/3

# Step 11: Using density and mean AD to calculate R (Clark & Evans, 1954)
meanAD_randomdistA <- 1/(2*sqrt(densityA))
meanAD_randomdistSM <- 1/(2*sqrt(densitySM))
meanAD_uniformA <- 1.0746/sqrt(densityA)
meanAD_uniformSM <- 1.0746/sqrt(densitySM)

R_aleixo <- meanAD_A/meanAD_randomdistA
R_santamarta <- meanAD_SM/meanAD_randomdistSM
R_random <- 1
R_uniformA <- meanAD_uniformA/meanAD_randomdistA

# Step 12: Alternative method for producing 3D PCA plots using coordinates
par(mfrow=c(1,1)) 

scatterA <- scatterplot3d(pcaA_x, pcaA_y, pcaA_z, tick.marks = TRUE, grid = TRUE, angle = 25, pch = 16, main = 'Aleixo Morning Chorus', color = 'black', xlim = c(-6,6), xlab = 'PC1 (42.4%)', ylim = c(-6,6), ylab = 'PC2 (22.2%)', zlim = c(-3,3), zlab = 'PC3 (17.9%)')
regplaneA <- lm(pcaA_x ~ pcaA_y + pcaA_z)
scatterA$plane3d(regplaneA)
scatterA$points3d(0, 0, 0, col='darkred', cex.symbols = 5, pch=16)

scatterSM <- scatterplot3d(pcaSM_x, pcaSM_y, pcaSM_z, tick.marks = TRUE, grid = TRUE, angle = 25, pch = 16, main = 'Santa Marta Morning Chorus', color = 'black', xlim = c(-6,6), xlab = 'PC1 (45.4%)', ylim = c(-6,6), ylab = 'PC2 (21.3%)', zlim = c(-4,3), zlab = 'PC3 (15.3%)')
regplaneSM <- lm(pcaSM_x ~ pcaSM_y + pcaSM_z)
scatterSM$plane3d(regplaneSM)
scatterSM$points3d(0, 0, 0, col='darkred', cex.symbols = 5, pch=16)

#normalized pca

par(mfrow=c(1,1)) 

scatterA <- scatterplot3d(normalize(pcaA_x), normalize(pcaA_y), normalize(pcaA_z), tick.marks = TRUE, grid = TRUE, angle = 25, pch = 16, main = 'Aleixo Morning Chorus', color = 'black', xlim = c(-0.3,0.3), xlab = 'PC1 (42.4%)', ylim = c(-0.3,0.3), ylab = 'PC2 (22.2%)', zlim = c(-0.3,0.3), zlab = 'PC3 (17.9%)')
regplaneA <- lm(normalize(pcaA_x) ~ normalize(pcaA_y) + normalize(pcaA_z))
scatterA$plane3d(regplaneA)
scatterA$points3d(0, 0, 0, col='darkred', cex.symbols = 5, pch=16)

# Step 13: Alternative method for producing 2D PCA plots
par(mfrow=c(1,1)) 
par(mar=c(5,5,4,4))
scatterA2D <- plot(pcaA_x, pcaA_y, main = 'Aleixo Morning Chorus', pch = 16, col = 'black', xlim = c(-5,5), xlab = 'PC1 (42.7%)', ylim = c(-2,6), ylab = 'PC2 (21.7%)')
scatterSM2D <- plot(pcaSM_x, pcaSM_y, main = 'Santa Marta Morning Chorus', pch = 16, col = 'black', xlim = c(-5,5), xlab = 'PC1 (42.7%)', ylim = c(-6,2), ylab = 'PC2 (21.7%)')

# Step 14: Interpreting the principal components

# # The correlation matrix produced by the principal component analysis
# # can be used to interpret the principal components. The seven percentages
# # placed in each vector below represent the strength of the correlation 
# # between the principal component and the seven acoustic variables.

eigenvectors <- c('Low Freq.', 'High Freq.', 'Central Freq.', 'Duration', 'Bandwidth', 'No. notes', 'Pace')

PC1eigenvectors_aleixo <- soundA.pca$rotation[1:7]^2
PC2eigenvectors_aleixo <- soundA.pca$rotation[8:14]^2
PC3eigenvectors_aleixo <- soundA.pca$rotation[15:21]^2
PC4eigenvectors_aleixo <- soundA.pca$rotation[22:28]^2

PC1eigenvectors_santamarta <- soundSM.pca$rotation[1:7]^2
PC2eigenvectors_santamarta <- soundSM.pca$rotation[8:14]^2
PC3eigenvectors_santamarta <- soundSM.pca$rotation[15:21]^2
PC4eigenvectors_santamarta <- soundSM.pca$rotation[22:28]^2

# Step 15: Bar plots for principal component interpretation
par(mfrow=c(1,1)) 

barplot(PC1eigenvectors_aleixo, main = 'Aleixo Principal Component 1', col = 'steelblue', ylim = c(0,0.5), cex.names = 0.7, space = 0.4, names = eigenvectors)
barplot(PC1eigenvectors_santamarta, main = 'Santa Marta Principal Component 1', col = 'steelblue', ylim = c(0,0.5), cex.names = 0.7, space = 0.4, names = eigenvectors)

barplot(PC2eigenvectors_aleixo, main = 'Aleixo Principal Component 2', col = 'darkgreen', ylim = c(0,0.7), cex.names = 0.7, space = 0.4, names = eigenvectors)
barplot(PC2eigenvectors_santamarta, main = 'Santa Marta Principal Component 2', col = 'darkgreen', ylim = c(0,0.7), cex.names = 0.7, space = 0.4, names = eigenvectors)

barplot(PC3eigenvectors_aleixo, main = 'Aleixo Principal Component 3', col = 'darkred', ylim = c(0,0.7), cex.names = 0.7, space = 0.4, names = eigenvectors)
barplot(PC3eigenvectors_santamarta, main = 'Santa Marta Principal Component 3', col = 'darkred', ylim = c(0,0.7), cex.names = 0.7, space = 0.4, names = eigenvectors)

# Step 16: Pie charts for principal component interpretation
piecolorlist <- c('darkred', 'steelblue', 'darkgreen', 'gray', 'purple', 'orange', 'pink')
par(mfrow=c(1,2)) 

pie(PC1eigenvectors_aleixo, main = 'Aleixo Principal Component 1', eigenvectors, col = piecolorlist)
pie(PC1eigenvectors_santamarta, main = 'Santa Marta Principal Component 1', eigenvectors, col = piecolorlist)

pie(PC2eigenvectors_aleixo, main = 'Aleixo Principal Component 2', eigenvectors, col = piecolorlist)
pie(PC2eigenvectors_santamarta, main = 'Santa Marta Principal Component 2', eigenvectors, col = piecolorlist)

pie(PC3eigenvectors_aleixo, main = 'Aleixo Principal Component 3', eigenvectors, col = piecolorlist)
pie(PC3eigenvectors_santamarta, main = 'Santa Marta Principal Component 3', eigenvectors, col = piecolorlist)

pie(PC4eigenvectors_aleixo, main = 'Aleixo Principal Component 4', eigenvectors, col = piecolorlist)
pie(PC4eigenvectors_santamarta, main = 'Santa Marta Principal Component 4', eigenvectors, col = piecolorlist)

# Step 17: Testing for multivariate normality in the PCA plot results
shapirodataA <- t(pcaAresults[1:3])
shapirodataSM <- t(pcaSMresults[1:3])

mshapiro.test(shapirodataA)
mshapiro.test(shapirodataSM)

testtable_A <- data.frame("Aleixo", length(soundA$Species), meanAD_A, sdAD_A, densityA, R_aleixo, mshapiro.test(shapirodataA)$statistic, mshapiro.test(shapirodataA)$p.value)
testtable_SM <- data.frame("Santa Marta", length(soundSM$Species), meanAD_SM, sdAD_SM, densitySM, R_santamarta, mshapiro.test(shapirodataSM)$statistic, mshapiro.test(shapirodataSM)$p.value)
testtablenames <- c('Study Site','N', 'Mean AD', 'St. Dev', 'Density', 'R', 'W', 'p')  
names(testtable_A) <- testtablenames
names(testtable_SM) <- testtablenames

# Step 18: Running linear regressions for acoustic distance (AD) against temporal overlap (TO)

# # First for Aleixo...
# # Creating a loop to make AD/TO dataframes for each species
distanceDFs_A <- list()

for (i in 1:57) {
  distanceDFs_A[[i]] <- data.frame(soundA$Species, unlist(ADmatrix_A[i]), unlist(percentoverlapA[i]))
  names(distanceDFs_A[[i]]) <- c('Species', 'Acoustic Distance', 'Temporal Overlap')
  distanceDFs_A[[i]] <- filter(distanceDFs_A[[i]], !distanceDFs_A[[i]]$`Temporal Overlap`==0) # removing species pairs with no temporal overlap
  distanceDFs_A[[i]] <- distanceDFs_A[[i]][-c(i),] # removing species pairings with itself
  
}

# # Removing any species which has only temporal distances of zero
zerodistances_A <- which(sapply(distanceDFs_A, nrow) < 1)
distanceDFs_A <- distanceDFs_A[-zerodistances_A]

# # Producing vectors of model coefficients (or slopes) and RSS values for each species
regslopes_A <- numeric()
RSS_A <- numeric()

for (i in 1:40) {
  x <- as.data.frame(distanceDFs_A[i])
  y <- lm(unlist(x[3]) ~ unlist(x[2]))$coefficients[2]
  z <- summary(lm(unlist(x[3]) ~ unlist(x[2])))$r.squared
  regslopes_A <- append(regslopes_A, y)
  RSS_A <- append(RSS_A, z)
}

# # Combining important model information into one data frame for Aleixo
RSS_A[is.na(RSS_A)] <- 0 
regtable_A <- data.frame(soundA$Species[-zerodistances_A], RSS_A, regslopes_A)
names(regtable_A) <- c("Species", "R-squared", "Coefficient")
regtable_A <- regtable_A %>% mutate_if(is.numeric, round, digits=3)

# # poscor/negcor are the proportion of species that exhibited a
# # positive/negative association between AD and TO
poscor_A <- sum(regtable_A$Coefficient > 0)/40
negcor_A <- sum(regtable_A$Coefficient < 0)/40

# # Repeating same process for Santa Marta
distanceDFs_SM <- list()

for (i in 1:29) {
  distanceDFs_SM[[i]] <- data.frame(soundSM$Species, unlist(ADmatrix_SM[i]), unlist(percentoverlapSM[i]))
  names(distanceDFs_SM[[i]]) <- c('Species', 'Acoustic Distance', 'Temporal Overlap')
  distanceDFs_SM[[i]] <- filter(distanceDFs_SM[[i]], !distanceDFs_SM[[i]]$`Temporal Overlap`==0)
  distanceDFs_SM[[i]] <- distanceDFs_SM[[i]][-c(i),]
  
}

zerodistances_SM <- which(sapply(distanceDFs_SM, nrow) < 1)
distanceDFs_SM <- distanceDFs_SM[-zerodistances_SM]

regslopes_SM <- numeric()
RSS_SM <- numeric()

for (i in 1:22) {
  x <- as.data.frame(distanceDFs_SM[i])
  y <- lm(unlist(x[3]) ~ unlist(x[2]))$coefficients[2]
  z <- summary(lm(unlist(x[3]) ~ unlist(x[2])))$r.squared
  regslopes_SM <- append(regslopes_SM, y)
  RSS_SM <- append(RSS_SM, z)
}

RSS_SM[is.na(RSS_SM)] <- 0

regtable_SM <- data.frame(soundSM$Species[-zerodistances_SM], RSS_SM, regslopes_SM)
names(regtable_SM) <- c("Species", "R-squared", "Coefficient")
regtable_SM <- regtable_SM %>% mutate_if(is.numeric, round, digits=3)

poscor_SM <- sum(regtable_SM$Coefficient > 0)/22
negcor_SM <- sum(regtable_SM$Coefficient < 0)/22

# # Trying normalized data

normalize <- function(x) {
  return(x / sqrt(sum(x^2)))
  }
soundA_normalized <- data.frame(normalize(as.numeric(unlist((soundA[2])))),normalize(as.numeric(unlist((soundA[3])))),normalize(as.numeric(unlist((soundA[4])))),normalize(as.numeric(unlist((soundA[5])))), normalize(as.numeric(unlist((soundA[6])))), normalize(as.numeric(unlist((soundA[7])))), normalize(as.numeric(unlist((soundA[8])))))

plot(normalize(meantempdistA)~normalize(meandistanceA), main = 'Aleixo Morning Chorus', xlab = 'Acoustic Distance to Mean Song', ylab = 'Temporal Distance to Mean Song')
abline(lm(normalize(meantempdistA)~normalize(meandistanceA)), col='red')
text(normalize(meantempdistA)~normalize(meandistanceA), pos = 2, cex = 0.5)

plot(normalize(meantempdistSM)~normalize(meandistanceSM), main = 'Santa Marta Morning Chorus', xlab = 'Acoustic Distance to Mean Song', ylab = 'Temporal Distance to Mean Song')
abline(lm(normalize(meantempdistSM)~normalize(meandistanceSM)), col='red')
text(normalize(meantempdistSM)~normalize(meandistanceSM), pos = 2, cex = 0.5)

unlist(speciesA[-(which(occsumsA<10))])


# Making data frame with temporal and acoustic distances from community centre
distancesA <- data.frame(soundA$Species, normalize(meantempdistA), normalize(meandistanceA))
names(distancesA) <- c("Species", "Temporal Distance", "Acoustic Distance")

distancesSM <- data.frame(soundSM$Species, normalize(meantempdistSM), normalize(meandistanceSM))
names(distancesSM) <- c("Species", "Temporal Distance", "Acoustic Distance")

write.table(format(distancesA, digits=2), file = "distancesA.txt", quote = FALSE, sep = ",")
write.table(format(distancesSM, digits=2), file = "distancesSM.txt", quote = FALSE, sep = ",")
